# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  ctrip-helper
# FileName:     libs.py
# Description:  TODO
# Author:       mfkifhss2023
# CreateDate:   2024/05/26
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""
import time
import logging
import typing as t
from enum import Enum
from copy import deepcopy
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement
from ctrip_helper.utils import get_current_datetime_int_str, join_path, get_image_dir, get_log_dir
from selenium.common.exceptions import NoSuchElementException, TimeoutException, ElementNotInteractableException

logger = logging.getLogger("root")


class EnumMetaClass(Enum):

    @classmethod
    def values(cls) -> list:
        return [x.value for x in cls]

    @classmethod
    def keys(cls) -> list:
        return [x.name for x in cls]

    @classmethod
    def get(cls, key: str) -> t.Any:
        if key.upper() in cls.keys():
            return getattr(cls, key.upper()).value
        elif key.lower() in cls.keys():
            return getattr(cls, key.lower()).value
        else:
            return None

    @classmethod
    def items(cls) -> t.List:
        return [(x.name, x.value) for x in cls]


class Locator(Enum):
    id = By.ID
    name = By.NAME
    xpath = By.XPATH
    tag_name = By.TAG_NAME
    link_text = By.LINK_TEXT
    class_name = By.CLASS_NAME
    css_selector = By.CSS_SELECTOR
    partial_link_text = By.PARTIAL_LINK_TEXT


def is_exist_element(driver: webdriver, locator: str, regx: str, loop: int, sleep: float,
                     is_ignore: bool = True, is_log_output: bool = True) -> bool:
    is_exist = False
    locator = getattr(getattr(Locator, locator, object), "value", "")
    for i in range(loop):
        try:
            # 根据实际情况定位按钮元素
            element = driver.find_element(locator, regx)
            if element:
                is_exist = True
                break
        except (NoSuchElementException,):
            if is_log_output is True:
                logger.error("Element Not Found")
            if is_ignore is False:
                raise NoSuchElementException()
        except (TimeoutException,):
            if is_log_output is True:
                logger.error("Element found timeout")
            if is_ignore is False:
                raise TimeoutException()
        except Exception as e:
            err_str = "通过选择器：{}，表达式: {}，判断元素是否存在失败".format(locator, regx)
            e_slice = str(e).split("Message:")
            if e_slice[0]:
                err_str = err_str + "，error: {}".format(e_slice[0])
            if is_log_output is True:
                logger.error(err_str)
            if is_ignore is False:
                raise OverflowError("Element found failed, reason: {}".format(err_str))
        if sleep > 0:
            time.sleep(sleep)
    return is_exist


def get_element(driver: webdriver, locator: str, regx: str, loop: int, sleep: float, **kwargs) -> WebElement:
    element = None
    is_ignore = kwargs.get("is_ignore", True)
    is_log_output = kwargs.get("is_log_output", True)
    locator = getattr(getattr(Locator, locator, object), "value", "")
    for i in range(loop):
        try:
            # 根据实际情况定位按钮元素
            element = driver.find_element(locator, regx)
            if element:
                return element
        except (NoSuchElementException,):
            if is_log_output is True:
                logger.error("Element Not Found")
            if is_ignore is False:
                raise NoSuchElementException()
        except (TimeoutException,):
            if is_log_output is True:
                logger.error("Element found timeout")
            if is_ignore is False:
                raise TimeoutException()
        except Exception as e:
            err_str = "通过选择器：{}，表达式: {}，获取元素失败".format(locator, regx)
            e_slice = str(e).split("Message:")
            if e_slice[0]:
                err_str = err_str + "，error: {}".format(e_slice[0])
            if is_log_output is True:
                logger.error(err_str)
            if is_ignore is False:
                raise OverflowError("Element found failed, reason: {}".format(err_str))
        if sleep > 0:
            time.sleep(sleep)
    return element


def click(element: WebElement, loop: int, sleep: float, **kwargs) -> bool:
    is_ignore = kwargs.get("is_ignore", True)
    is_log_output = kwargs.get("is_log_output", True)
    for _ in range(loop):
        try:
            element.click()
            return True
        except (ElementNotInteractableException,):
            if is_log_output is True:
                logger.error("Waiting for element to become interactive")
            if is_ignore is False:
                raise RuntimeError("Waiting for element to become interactive")
        except Exception as e:
            e_slice = str(e).split("Message:")
            if is_log_output is True:
                logger.error(e_slice[0])
            if is_ignore is False:
                raise OverflowError("Element click failed, reason: {}".format(e_slice[0]))
        if sleep > 0:
            time.sleep(sleep)
    return False


def send_keys(element: WebElement, value: str, loop: int, sleep: float, **kwargs) -> bool:
    is_ignore = kwargs.get("is_ignore", True)
    is_log_output = kwargs.get("is_log_output", True)
    for _ in range(loop):
        try:
            element.send_keys(value)
            return True
        except (ElementNotInteractableException,):
            if is_log_output is True:
                logger.error("Waiting for element to become interactive")
            if is_ignore is False:
                raise RuntimeError("Waiting for element to become interactive")
        except Exception as e:
            e_slice = str(e).split("Message:")
            if is_log_output is True:
                logger.error(e_slice[0])
            if is_ignore is False:
                raise OverflowError("Element click failed, reason: {}".format(e_slice[0]))
        if sleep > 0:
            time.sleep(sleep)
    return False


def switch_to_window(driver: webdriver, origin_windows: list, loop: int, sleep: float) -> bool:
    for i in range(loop):
        # 获取所有的窗口句柄
        all_windows = driver.window_handles
        diff = set(all_windows).difference(set(origin_windows))
        if len(diff) > 0:
            new_window = diff.pop()
            driver.switch_to.window(new_window)
            return True
        if sleep > 0:
            time.sleep(sleep)
    return False


def close_other_windows(driver: webdriver, origin_window: str) -> bool:
    # 获取所有的窗口句柄
    all_windows = driver.window_handles
    diff = list(set(all_windows).difference({origin_window}))
    if diff:
        for index, window in enumerate(diff):
            driver.switch_to.window(window)
            driver.close()
            if index == len(diff) - 1:
                driver.switch_to.window(origin_window)
                return True
    else:
        return True


def is_switch_latest_page(driver: webdriver, latest_page: str, loop: int = 1, sleep: float = 0) -> tuple:
    current_url = ""
    for _ in range(loop):
        current_url = driver.current_url
        if current_url.startswith(latest_page) is True:
            return True, current_url
        if sleep > 0:
            time.sleep(sleep)
    return False, current_url


def get_screenshot(driver: webdriver, file_name: str = None, **kwargs) -> tuple:
    """获取当前截屏"""
    is_ignore = kwargs.get("is_ignore", True)
    is_log_output = kwargs.get("is_log_output", True)
    screenshot_file_name, screenshot_file_suffix = None, get_current_datetime_int_str()
    try:
        if file_name is None:
            file_name = "screenshot"
        else:
            file_name_slice = file_name.split(".")
            if len(file_name_slice) > 1:
                file_name = file_name_slice[0]
        iname_file_name = "{}_{}.png".format(file_name, screenshot_file_suffix)
        image_path = get_image_dir()
        screenshot_file_name = join_path([image_path, iname_file_name])
        driver.save_screenshot(screenshot_file_name)
    except Exception as e:
        err_str = "获取当前页面截屏失败"
        e_slice = str(e).split("Message:")
        if len(e_slice) > 0:
            err_str = err_str + "，原因：{}".format(e_slice[0])
        if is_log_output is True:
            logger.error(err_str)
        if is_ignore is False:
            raise OverflowError(err_str)
    return screenshot_file_name, screenshot_file_suffix


def exception_html_save_to_txt(driver: webdriver, file_name: str = None, **kwargs) -> tuple:
    is_ignore = kwargs.get("is_ignore", True)
    is_log_output = kwargs.get("is_log_output", True)
    exception_file_name, exception_file_suffix = None, get_current_datetime_int_str()
    try:
        if file_name is None:
            file_name = "exception"
        else:
            file_name_slice = file_name.split(".")
            if len(file_name_slice) > 1:
                file_name = file_name_slice[0]
        html_file_name = "{}_{}.html".format(file_name, exception_file_suffix)
        log_path = get_log_dir()
        exception_file_name = join_path([log_path, html_file_name])
        with open(exception_file_name, "w", encoding="utf-8") as file:
            file.write(driver.page_source)
    except Exception as e:
        err_str = "页面异常内容保存至文本文件出错"
        e_slice = str(e).split("Message:")
        if len(e_slice) > 0:
            err_str = err_str + "，原因：{}".format(e_slice[0])
        if is_log_output is True:
            logger.error(err_str)
        if is_ignore is False:
            raise OverflowError(err_str)
    return exception_file_name, exception_file_suffix



# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  ctrip-helper
# FileName:     config.py
# Description:  TODO
# Author:       mfkifhss2023
# CreateDate:   2024/05/26
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""

common_headers = {
    'origin': "https://flights.ctrip.com",
    'content-type': "application/json;charset=UTF-8",
    'accept': "application/json",
    'cache-control': "no-cache",
    "accept-encoding": "gzip, deflate, br, zstd",
    "accept-language": "zh-CN,zh;q=0.9",
    "Sec-Ch-Ua-Platform": "Windows",
    "sec-ch-ua": '"Google Chrome";v="125", "Chromium";v="125", "Not.A/Brand";v="24"',
    "sec-ch-ua-mobile": "?0",
    "sec-fetch-dest": "empty",
    "sec-fetch-site": "same-origin",
    "sec-fetch-mode": "cors",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, " +
                  "like Gecko) Chrome/92.0.4515.131 Safari/537.36 Edg/92.0.902.67"
}

url_map = {
    'domain': 'flights.ctrip.com',
    "protocol": "https",
    "city_code": "/online/api/poi/get",
    # 获取法定节假日
    "international_holiday": "/international/search/api/base/getHoliday",
    # 获取国家编码
    "country_code": "/international/search/restapi/soa2/15095/json/flightCityPageData",
    # 获取航线的航段信息，参数：/oneway-bjs-sel?_=1&depdate=2021-12-31&cabin=y_s&adult=1&containstax=1
    "flight_segments": "/international/search/api/flightlist/oneway",
    # 获取航线的航班列表
    "airline_list": "/international/search/api/search/batchSearch",
    # 机票订单查询首页
    "order_query_home_with_flight": "https://my.ctrip.com/myinfo/flight",
    # 订单详情页，?oid=32665635142
    "flight_order_details": "https://flights.ctrip.com/online/orderdetail/index",
    # 携程机票安全支付界面，?tradeNo=20240529144727TP1193583667800373067821
    "safe_payment_home": "https://secure.ctrip.com/webapp/payment8/index",
    # 易宝会员支付账号输入界面
    "yeepay2b_cash_desk": "https://cashdesk.yeepay.com/bc-cashier/bcnewpc/request",
    # 登录页
    "ctrip_login_prefix": "https://passport.ctrip.com/user/login",
    "ctrip_login_suffix": "{}?BackUrl=https%3A%2F%2Fwww.ctrip.com%2F#ctm_ref=c_ph_login_buttom",
}

airline_request_params_template = {
    "adultCount": 1,
    "childCount": 0,
    "infantCount": 0,
    "flightWay": "S",  # OW: 单程, S:
    "cabin": "Y_S",
    "scope": "d",
    "extensionAttributes": {
        "LoggingSampling": False,
        "isFlightIntlNewUser": False
    },
    "segmentNo": 1,
    "transactionID": "13960086675943ef869fae36c6818883",
    "flightSegments": list(),  # 航段
    "directFlight": False,  # 是否为直飞
    "extGlobalSwitches": {
        "useAllRecommendSwitch": True,
        "unfoldPriceListSwitch": True
    },
    "noRecommend": False  # 是否要推荐产品
}

flight_segment_template = {
    "departureCityCode": "",
    "arrivalCityCode": "",
    "departureCityName": "",
    "arrivalCityName": "",
    "departureDate": "",
    "departureCountryId": 1,
    "departureCountryName": "中国",
    "departureCountryCode": "CN",
    "departureProvinceId": 0,
    "departureCityId": 0,
    "arrivalCountryId": 1,
    "arrivalCountryName": "中国",
    "arrivalCountryCode": "CN",
    "arrivalProvinceId": 0,
    "arrivalCityId": 0,
    "departureCityTimeZone": 480,
    "arrivalCityTimeZone": 480,
    "timeZone": 480
}



# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  ctrip-helper
# FileName:     desktop_ui.py
# Description:  TODO
# Author:       mfkifhss2023
# CreateDate:   2024/05/29
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""
import re
import time
from enum import Enum
from decimal import Decimal
from selenium import webdriver
from ctrip_helper.libs import logger
from ctrip_helper.config import url_map
from selenium.webdriver.common.keys import Keys
from ctrip_helper.utils import is_later_than_current_time
from ctrip_helper.libs import get_element, send_keys, is_switch_latest_page


class UILocatorRegx(Enum):
    all_orders_select_box = {"locator": "xpath",
                             "regx": '//ul[@class="fix_myctrip_order_layer"]//div[@class="select-box"]'}
    order_id_input_box = {"locator": "xpath", "regx": '//div[@class="order_inquiry"]//input[@id="searchBookingNum"]'}
    order_query_button = {"locator": "xpath", "regx": '//div[@class="order_inquiry"]//button[@class="btn_sel"]'}
    order_status_with_list = {"locator": "xpath",
                              "regx": '//ul[@class="t_body"]//span[@class="order-price-status-title"]'}
    order_status_sub_title = {"locator": "xpath",
                              "regx": '//div[contains(@data-testid, "orderstatus_StatusSubTitleWrap")]'}
    order_status_booking_amount = {"locator": "xpath",
                                   "regx": '//div[@data-testid="fbu_PaymentWrapper"]//span[@data-testid="false"]'}
    order_to_payment = {"locator": "xpath", "regx": '//div[@id="button-group"]//a[@data-ubt-v="主订单支付"]'}
    order_to_cancel = {"locator": "xpath", "regx": '//div[@id="button-group"]//a[@data-ubt-v="取消订单"]'}
    order_continue_cancel = {"locator": "xpath", "regx": '//div[@class="ant-modal"]//div[@data-testid="继续取消"]'}
    order_cancel_got_it = {"locator": "xpath", "regx": '//div[@class="ant-modal"]//div[@data-testid="知道了"]'}
    order_payment_amount = {"locator": "xpath", "regx": '//div[@class="m-order-amount"]//div[@class="m-order-money"]'}
    wallet_disable = {"locator": "xpath", "regx": '//div[@class="walelt-card"]//div[@class="lin-crd-disabled"]'}
    wallet_balance = {"locator": "xpath", "regx": '//div[@class="walelt-card"]//div[contains(@class, "crdlast")]'}
    use_wallet = {"locator": "xpath", "regx": '//div[@class="walelt-card"]//input[@class="am-switch-checkbox"]'}
    wallet_immediately_payment = {"locator": "xpath",
                                  "regx": '//div[@class="wallet_pay_button"]//button[@type="gradient"]'}
    use_yeepay2b_pyament = {"locator": "xpath",
                            "regx": '//div[@class="pay_way_container"]//div[contains(text(), "易宝会员支付")]'}
    yeepay2b_immediately_payment = {"locator": "xpath",
                                    "regx": '//div[@class="pay_btn_container"]//div[@class="trip-pay-btn-text"]'}
    password_input_box = {"locator": "xpath",
                          "regx": '//div[@class="verify-password-box"]//input[contains(@class, "ant-input")]'}
    yeepay2b_accout_input_box = {"locator": "xpath",
                                 "regx": '//div[@class="account-pay-main"]//input[@name="userAccount"]'}
    yeepay2b_password_input_box = {"locator": "xpath",
                                   "regx": '//div[@class="account-pay-main"]//input[@name="tradePassword"]'}
    yeepay2b_payment_next_button = {"locator": "xpath",
                                    "regx": '//div[@class="account-pay-main"]//button[@id="passPayButton"]'}
    is_login_button = {
        "locator": "xpath",
        "regx": '//div[@class="tl_nfes_home_header_login_wrapper_siwkn"]//button[contains(@aria-label, "我的账户")]'
    }
    is_login_span = {"locator": "xpath", "regx": '//div[@class="loginbar"]//span[@class="ctrip-username"]'}
    account_input = {"locator": "xpath",
                     "regx": '//div[@data-testid="accountPanel"]//input[@data-testid="accountNameInput"]'}
    password_input = {"locator": "xpath",
                      "regx": '//div[@data-testid="accountPanel"]//input[@data-testid="passwordInput"]'}
    service_agreement = {"locator": "xpath",
                         "regx": '//div[@data-testid="agreementList"]//label[@for="checkboxAgreementInput"]'}
    login_button = {"locator": "xpath", "regx": '//div[@data-testid="accountPanel"]//input[@data-testid="loginButton"]'}
    slider_verify = {"locator": "xpath", "regx": '//div[@data-testid="captcha"]/div'}
    more_bank_card = {"locator": "xpath", "regx": '//div[@class="payment_trip_pay_container"]//div[@class="more_card"]'}
    bank_card = {"locator": "xpath", "regx": '//div[@class="payment_trip_pay_container"]//div[contains(text(), "{}")]'}
    bank_card_payment = {"locator": "xpath",
                         "regx": '//div[@class="payment_trip_pay_container"]//div[@class="trip-pay-btn-text"]'}
    yeepay2b_payment_success = {"locator": "xpath", "regx": '//div[@class="success-layout"]//p[@class="pay-success"]'}
    submit_payment_result = {"locator": "xpath",
                             "regx": '//div[contains(@class,"cds-dialog-content")]//div[contains(text(), "支付确认")]'}
    wallet_payment_result_status = {
        "locator": "xpath",
        "regx": '//div[@data-testid="orderstatus_StatusWrap"]' +
                '//div[contains(@data-testid, "orderstatus_StatusTitle_colorValue")]'
    }


class SeleniumApi(object):

    @classmethod
    def is_login(cls, driver: webdriver, username: str, platform: str, loop: int = 1, sleep: float = 0,
                 **kwargs) -> bool:
        """是否已登录"""
        is_login_button = get_element(
            driver=driver, locator=UILocatorRegx.is_login_button.value.get("locator"),
            regx=UILocatorRegx.is_login_button.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if is_login_button:
            logger.info("{}平台用户<{}>已登录".format(platform, username))
            return True
        is_login_span = get_element(
            driver=driver, locator=UILocatorRegx.is_login_span.value.get("locator"),
            regx=UILocatorRegx.is_login_span.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if is_login_span:
            logger.info("{}平台用户<{}>已登录".format(platform, username))
            return True
        else:
            return False

    @classmethod
    def open_login_page(cls, driver: webdriver, sleep: float = 0) -> None:
        ctrip_login_prefix = url_map.get('ctrip_login_prefix')
        ctrip_login_suffix = url_map.get('ctrip_login_suffix')
        login_url = "{}{}".format(ctrip_login_prefix, ctrip_login_suffix)
        driver.get(login_url)
        if sleep > 0:
            time.sleep(sleep)
        logger.info("打开携程网页版登录页")

    @classmethod
    def enter_user_name(cls, driver: webdriver, username: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        input_box = get_element(
            driver=driver, locator=UILocatorRegx.account_input.value.get("locator"),
            regx=UILocatorRegx.account_input.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if input_box:
            # 模拟键盘操作清空输入框内容
            input_box.send_keys(Keys.CONTROL + "a")  # 选中输入框中的所有内容
            input_box.send_keys(Keys.BACKSPACE)  # 删除选中的内容
            input_box.send_keys('{}'.format(username))
            logger.info("输入登录账号：{}".format(username))
            return True
        else:
            return False

    @classmethod
    def enter_user_password(cls, driver: webdriver, password: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        input_box = get_element(
            driver=driver, locator=UILocatorRegx.password_input.value.get("locator"),
            regx=UILocatorRegx.password_input.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if input_box:
            # 模拟键盘操作清空输入框内容
            input_box.send_keys(Keys.CONTROL + "a")  # 选中输入框中的所有内容
            input_box.send_keys(Keys.BACKSPACE)  # 删除选中的内容
            input_box.send_keys('{}'.format(password))
            logger.info("输入登录密码：{}".format(password))
            return True
        else:
            return False

    @classmethod
    def click_service_agreement(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        checkbox = get_element(
            driver=driver, locator=UILocatorRegx.service_agreement.value.get("locator"),
            regx=UILocatorRegx.service_agreement.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if checkbox:
            checkbox.click()
            logger.info("选中【阅读并同意携程的服务协议和个人信息保护政策】")
            return True
        else:
            return False

    @classmethod
    def click_login(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        login_button = get_element(
            driver=driver, locator=UILocatorRegx.login_button.value.get("locator"),
            regx=UILocatorRegx.login_button.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if login_button:
            login_button.click()
            logger.info("点击登录页面的【登录】按钮")
            return True
        else:
            return False

    @classmethod
    def is_exist_slider_verify(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        current_url = driver.current_url
        slider_verify = get_element(
            driver=driver, locator=UILocatorRegx.slider_verify.value.get("locator"),
            regx=UILocatorRegx.slider_verify.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if slider_verify:
            logger.info("当前页：{} 出现了滑块验证码".format(current_url))
            return True
        else:
            return False

    @classmethod
    def open_order_query_home_with_flight(cls, driver: webdriver, sleep: float = 0) -> None:
        driver.get(url_map.get('order_query_home_with_flight'))
        if sleep > 0:
            time.sleep(sleep)
        logger.info("打开机票订单查询首页")

    @classmethod
    def click_more_filter_conditions(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        more_filter_expand = get_element(
            driver=driver, locator=UILocatorRegx.all_orders_select_box.value.get("locator"),
            regx=UILocatorRegx.all_orders_select_box.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if more_filter_expand:
            more_filter_expand.click()
            logger.info("选择【更多筛选条件】，展开条件列表")
            return True
        else:
            return False

    @classmethod
    def enter_order_id(cls, driver: webdriver, order_id: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        input_box = get_element(
            driver=driver, locator=UILocatorRegx.order_id_input_box.value.get("locator"),
            regx=UILocatorRegx.order_id_input_box.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if input_box:
            # 模拟键盘操作清空输入框内容
            input_box.send_keys(Keys.CONTROL + "a")  # 选中输入框中的所有内容
            input_box.send_keys(Keys.BACKSPACE)  # 删除选中的内容
            input_box.send_keys('{}'.format(order_id))
            logger.info("输入查询订单号：{}".format(order_id))
            return True
        else:
            return False

    @classmethod
    def click_order_query(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        order_query_button = get_element(
            driver=driver, locator=UILocatorRegx.order_query_button.value.get("locator"),
            regx=UILocatorRegx.order_query_button.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if order_query_button:
            order_query_button.click()
            logger.info("点击【查询】按钮")
            return True
        else:
            return False

    @classmethod
    def get_order_status(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> str:
        order_status = ""
        order_status_element = get_element(
            driver=driver, locator=UILocatorRegx.order_status_with_list.value.get("locator"),
            regx=UILocatorRegx.order_status_with_list.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if order_status_element:
            order_status = order_status_element.text.strip()
            logger.info("获取订单的状态为：{}".format(order_status))
        return order_status

    @classmethod
    def click_order_status_with_query_list(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        order_status_element = get_element(
            driver=driver, locator=UILocatorRegx.order_status_with_list.value.get("locator"),
            regx=UILocatorRegx.order_status_with_list.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if order_status_element:
            order_status_element.click()
            logger.info("点击订单状态，进入订单详情界面")
            return True
        else:
            return False

    @classmethod
    def click_to_payment(cls, driver: webdriver, order_id: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        local_url = "{}?oid={}".format(url_map.get("flight_order_details"), order_id)
        is_success, current_url = is_switch_latest_page(driver=driver, latest_page=local_url, loop=loop, sleep=sleep)
        if is_success is True:
            to_payment_button = get_element(
                driver=driver, locator=UILocatorRegx.order_to_payment.value.get("locator"),
                regx=UILocatorRegx.order_to_payment.value.get("regx"), loop=loop, sleep=sleep, **kwargs
            )
            if to_payment_button:
                to_payment_button.click()
                logger.info("点击【去支付】，进入安全支付界面")
                return True
            else:
                return False
        else:
            logger.warn("当前页: {}，不是本次要支付的订单: {} 的详情页".format(current_url, order_id))
            return False

    @classmethod
    def click_order_cancel(cls, driver: webdriver, order_id: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        local_url = "{}?oid={}".format(url_map.get("flight_order_details"), order_id)
        is_success, current_url = is_switch_latest_page(driver=driver, latest_page=local_url, loop=loop, sleep=sleep)
        if is_success is True:
            order_cancel_button = get_element(
                driver=driver, locator=UILocatorRegx.order_to_cancel.value.get("locator"),
                regx=UILocatorRegx.order_to_cancel.value.get("regx"), loop=loop, sleep=sleep, **kwargs
            )
            if order_cancel_button:
                order_cancel_button.click()
                logger.info("点击【取消订单】，接下来会出现【取消提示】小弹框")
                return True
            else:
                return False
        else:
            logger.warn("当前页: {}，不是本次要支付的订单: {} 的详情页".format(current_url, order_id))
            return False

    @classmethod
    def click_order_continue_cancel(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        continue_cance_button = get_element(
            driver=driver, locator=UILocatorRegx.order_continue_cancel.value.get("locator"),
            regx=UILocatorRegx.order_continue_cancel.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if continue_cance_button:
            continue_cance_button.click()
            logger.info("点击【继续取消】，接下来会出现【知道了】小弹框")
            return True
        else:
            return False

    @classmethod
    def is_order_cancel_success(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        got_it_button = get_element(
            driver=driver, locator=UILocatorRegx.order_cancel_got_it.value.get("locator"),
            regx=UILocatorRegx.order_cancel_got_it.value.get("regx"), sleep=sleep, loop=loop, **kwargs
        )
        if got_it_button:
            return True
        else:
            return False

    @classmethod
    def get_order_amount_with_safe_payment_home(cls, driver: webdriver, loop: int = 1, sleep: float = 0,
                                                **kwargs) -> Decimal:
        order_payment_amount = "0.00"
        local_url = url_map.get("safe_payment_home")
        is_success, current_url = is_switch_latest_page(driver=driver, latest_page=local_url, loop=loop, sleep=sleep)
        if is_success is True:
            order_payment_amount_element = get_element(
                driver=driver, locator=UILocatorRegx.order_payment_amount.value.get("locator"),
                regx=UILocatorRegx.order_payment_amount.value.get("regx"), loop=loop, sleep=sleep, **kwargs
            )
            if order_payment_amount_element:
                order_payment_amount = order_payment_amount_element.text.strip()
                logger.info("获取订单的支付金额为：{}".format(order_payment_amount))
        else:
            logger.warning("当前页：{} 不是订单安全支付页".format(current_url))
        return Decimal(order_payment_amount)

    @classmethod
    def is_cancel_order_with_payment_amount(cls, driver: webdriver, out_total_amount: str, amount_loss_limit: str,
                                            profit_cap: str, passenger_number: int, platform: str, loop: int = 1,
                                            sleep: float = 0, discount_amount: str = None, **kwargs) -> tuple:
        """在订单详情页，判断是否需要取消订单，检验支付金额是否满足政策要求"""
        flag = False
        remark = ""
        booking_amount_element = get_element(
            driver=driver, locator=UILocatorRegx.order_status_booking_amount.value.get("locator"),
            regx=UILocatorRegx.order_status_booking_amount.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if booking_amount_element:
            booking_amount_text = booking_amount_element.text.strip()
            # 使用 findall 获取所有匹配项
            amount_match = re.search(r"¥(\d+)", booking_amount_text)
            if amount_match:
                amount_str = amount_match.group(1)
                logger.warning("从{}订单获取的支付金额：{}，劲旅订单总价：{}".format(
                    platform, amount_str, out_total_amount
                ))
                # 预期订单利润
                ex_order_profit = Decimal(out_total_amount) - Decimal(amount_str)
                if discount_amount:
                    # 实际订单利润
                    ac_order_profit = ex_order_profit + Decimal(discount_amount)
                else:
                    ac_order_profit = ex_order_profit
                # 订单利润 < 0, 存在亏钱，与亏钱的下限进行比较
                if ac_order_profit < 0:
                    total = Decimal(amount_loss_limit) * passenger_number
                    if ac_order_profit + total < 0:
                        flag = True
                        remark = "订单亏钱{:.2f}太多，超过订单总下限值{}(单人下限{} * {}人)".format(
                            abs(ac_order_profit), total, amount_loss_limit, passenger_number
                        )
                        logger.warning(remark)
                        return flag, remark
                # 订单利润 >= 0, 存在毛利，与利润的上限进行比较
                else:
                    total = Decimal(profit_cap) * passenger_number
                    if ac_order_profit - total > 0:
                        flag = True
                        remark = "订单利润{:.2f}太高，超过订单总下限值{}(单人下限{} * {}人)".format(
                            ac_order_profit, total, profit_cap, passenger_number
                        )
                        logger.warning(remark)
                        return flag, remark
            else:
                remark = "从订单详情页面获取订单过期时间和订单金额有异常"
                logger.warning(remark)
        else:
            remark = "订单详情页面中未找到订单金额元素"
            logger.warning(remark)
        return flag, remark

    @classmethod
    def is_cancel_order_with_remaining_payment_time(cls, driver: webdriver, loop: int = 1, sleep: float = 0,
                                                    **kwargs) -> tuple:
        """在订单详情页，判断是否需要取消订单，判断订单的剩余支付时间"""
        flag = False
        remark = ""
        status_sub_title_element = get_element(
            driver=driver, locator=UILocatorRegx.order_status_sub_title.value.get("locator"),
            regx=UILocatorRegx.order_status_sub_title.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if status_sub_title_element:
            sub_title_text = status_sub_title_element.text.strip()
            # 使用 findall 获取所有匹配项
            time_match = re.search(r"(\d{2}:\d{2})", sub_title_text)
            if time_match:
                time_str = time_match.group(1)
                logger.info("从订单获取到的过期时间为：{}".format(time_str))
                minutes = 1
                is_later = is_later_than_current_time(time_str=time_str, minutes=minutes)
                if is_later is False:
                    flag = True
                    remark = "支付时间少于{}分钟".format(minutes)
                    logger.warning(remark)
                    return flag, remark
            else:
                remark = "从文案<{}>中没有获取订单过期时间".format(sub_title_text)
                logger.warning(remark)
        else:
            remark = "从订单详情页面中未找到订单过期时间元素"
            logger.warning(remark)
        return flag, remark

    @classmethod
    def is_wallet_disable(cls, driver: webdriver, order_id: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        wallet_disable_element = get_element(
            driver=driver, locator=UILocatorRegx.wallet_disable.value.get("locator"),
            regx=UILocatorRegx.wallet_disable.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if wallet_disable_element:
            logger.warning("当前订单: {} 钱包不可用，需要切换至其他支付方式".format(order_id))
            return True
        else:
            return False

    @classmethod
    def get_wallet_balance(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> Decimal:
        wallet_balance = "0.00"
        wallet_balance_element = get_element(
            driver=driver, locator=UILocatorRegx.wallet_balance.value.get("locator"),
            regx=UILocatorRegx.wallet_balance.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if wallet_balance_element:
            text = wallet_balance_element.text.strip()
            logger.info("获取账号礼品卡{}".format(text))
            wallet_balance = text.split("¥")[-1]
        return Decimal(wallet_balance)

    @classmethod
    def click_use_wallet(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        use_wallet_switch = get_element(
            driver=driver, locator=UILocatorRegx.use_wallet.value.get("locator"),
            regx=UILocatorRegx.use_wallet.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if use_wallet_switch:
            use_wallet_switch.click()
            logger.info("选择使用钱包支付")
            return True
        else:
            return False

    @classmethod
    def click_wallet_immediately_payment(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        immediately_payment_button = get_element(
            driver=driver, locator=UILocatorRegx.wallet_immediately_payment.value.get("locator"),
            regx=UILocatorRegx.wallet_immediately_payment.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if immediately_payment_button:
            immediately_payment_button.click()
            logger.info("点击钱包【立即支付】")
            return True
        else:
            return False

    @classmethod
    def click_use_yeepay2b(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        use_yeepay2b_switch = get_element(
            driver=driver, locator=UILocatorRegx.use_yeepay2b_pyament.value.get("locator"),
            regx=UILocatorRegx.use_yeepay2b_pyament.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if use_yeepay2b_switch:
            use_yeepay2b_switch.click()
            logger.info("选择使用易宝会员支付")
            return True
        else:
            return False

    @classmethod
    def click_yeepay2b_immediately_payment(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        immediately_payment_button = get_element(
            driver=driver, locator=UILocatorRegx.yeepay2b_immediately_payment.value.get("locator"),
            regx=UILocatorRegx.yeepay2b_immediately_payment.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if immediately_payment_button:
            immediately_payment_button.click()
            logger.info("点击【易宝支付】按钮")
            return True
        else:
            return False

    @classmethod
    def enter_payment_password(cls, driver: webdriver, password: str, loop: int = 1, sleep: float = 0,
                               **kwargs) -> bool:
        password_input_box = get_element(
            driver=driver, locator=UILocatorRegx.password_input_box.value.get("locator"),
            regx=UILocatorRegx.password_input_box.value.get("regx"), sleep=sleep, loop=loop, **kwargs
        )
        if password_input_box:
            if not isinstance(password, str):
                password = str(password)
            for char in password:
                is_success = send_keys(element=password_input_box, value=char, loop=loop, sleep=sleep, **kwargs)
                if is_success is False:
                    return False
                # 可选：添加一个短暂的延迟，模拟更接近人类的输入速度
                time.sleep(0.5)
            return True
        else:
            logger.warning("没有出现输入密码的弹框")
            return False

    @classmethod
    def is_wallet_payment_success(cls, driver: webdriver, platform: str, order_id: str, loop: int = 1, sleep: float = 0,
                                  **kwargs) -> bool:
        order_status_element = get_element(
            driver=driver, locator=UILocatorRegx.wallet_payment_result_status.value.get("locator"),
            regx=UILocatorRegx.wallet_payment_result_status.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if order_status_element:
            text = order_status_element.text.strip()
            if "出票中" in text or "已出票" in text:
                logger.info("{}订单：{}，钱包支付成功".format(platform, order_id))
                return True
            else:
                return False
        else:
            return False

    @classmethod
    def enter_yeepay2b_account(cls, driver: webdriver, account: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        local_url = url_map.get("yeepay2b_cash_desk")
        is_success, current_url = is_switch_latest_page(driver=driver, latest_page=local_url, loop=loop, sleep=sleep)
        if is_success is True:
            input_box = get_element(
                driver=driver, locator=UILocatorRegx.yeepay2b_accout_input_box.value.get("locator"),
                regx=UILocatorRegx.yeepay2b_accout_input_box.value.get("regx"), loop=loop, sleep=sleep, **kwargs
            )
            if input_box:
                # 模拟键盘操作清空输入框内容
                input_box.send_keys(Keys.CONTROL + "a")  # 选中输入框中的所有内容
                input_box.send_keys(Keys.BACKSPACE)  # 删除选中的内容
                input_box.send_keys('{}'.format(account))
                logger.info("输入的易宝会员账号：{}".format(account))
                return True
            else:
                return False
        else:
            logger.warn("当前页: {}，不是易宝支付收银台页".format(current_url))
            return False

    @classmethod
    def enter_yeepay2b_password(cls, driver: webdriver, password: str, loop: int = 1, sleep: float = 0,
                                **kwargs) -> bool:
        input_box = get_element(
            driver=driver, locator=UILocatorRegx.yeepay2b_password_input_box.value.get("locator"),
            regx=UILocatorRegx.yeepay2b_password_input_box.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if input_box:
            # 模拟键盘操作清空输入框内容
            input_box.send_keys(Keys.CONTROL + "a")  # 选中输入框中的所有内容
            input_box.send_keys(Keys.BACKSPACE)  # 删除选中的内容
            input_box.send_keys('{}'.format(password))
            logger.info("输入的易宝会员账号密码：{}".format(password))
            return True
        else:
            return False

    @classmethod
    def click_yeepay2b_payment_next(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        next_button = get_element(
            driver=driver, locator=UILocatorRegx.yeepay2b_payment_next_button.value.get("locator"),
            regx=UILocatorRegx.yeepay2b_payment_next_button.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if next_button:
            next_button.click()
            logger.info("易宝支付收银台界面点击【下一步】")
            return True
        else:
            return False

    @classmethod
    def is_yeepay2b_payment_success(cls, driver: webdriver, platform: str, order_id: str, loop: int = 1,
                                    sleep: float = 0, **kwargs) -> bool:
        payment_success_element = get_element(
            driver=driver, locator=UILocatorRegx.yeepay2b_payment_success.value.get("locator"),
            regx=UILocatorRegx.yeepay2b_payment_success.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if payment_success_element:
            logger.info("{}订单：{}，易宝会员支付成功".format(platform, order_id))
            return True
        submit_payment_result_element = get_element(
            driver=driver, locator=UILocatorRegx.submit_payment_result.value.get("locator"),
            regx=UILocatorRegx.submit_payment_result.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if submit_payment_result_element:
            logger.info("{}订单：{}，易宝会员支付成功".format(platform, order_id))
            return True
        else:
            return False

    @classmethod
    def click_more_bank_card(cls, driver: webdriver, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        more_bank_card_expand = get_element(
            driver=driver, locator=UILocatorRegx.more_bank_card.value.get("locator"),
            regx=UILocatorRegx.more_bank_card.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if more_bank_card_expand:
            more_bank_card_expand.click()
            logger.info("点击【更多银行卡】")
            return True
        else:
            return False

    @classmethod
    def click_bank_card(cls, driver: webdriver, payment_card: str, loop: int = 1, sleep: float = 0, **kwargs) -> bool:
        bank_card_regx = UILocatorRegx.bank_card.value.get("regx")
        bank_card_regx = bank_card_regx.format(payment_card)
        bank_card_element = get_element(
            driver=driver, locator=UILocatorRegx.more_bank_card.value.get("locator"),
            regx=bank_card_regx, loop=loop, sleep=sleep, **kwargs
        )
        if bank_card_element:
            bank_card_element.click()
            logger.info("点击选择【{}】银行卡".format(payment_card))
            return True
        else:
            return False

    @classmethod
    def click_bank_card_payment(cls, driver: webdriver, payment_card: str, loop: int = 1, sleep: float = 0,
                                **kwargs) -> bool:
        bank_card_payment_button = get_element(
            driver=driver, locator=UILocatorRegx.bank_card_payment.value.get("locator"),
            regx=UILocatorRegx.bank_card_payment.value.get("regx"), loop=loop, sleep=sleep, **kwargs
        )
        if bank_card_payment_button:
            bank_card_payment_button.click()
            logger.info("点击【银行卡支付】，接下来会出现短信验证码输入框")
            return True
        else:
            return False



# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  ctrip-helper
# FileName:     utils.py
# Description:  TODO
# Author:       mfkifhss2023
# CreateDate:   2024/05/26
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""
import re
import os
import sys
import json
import random
import hashlib
import typing as t
import urllib.parse
from datetime import datetime, timedelta

standard_date_format = "%Y-%m-%d %H:%M:%S"


def get_current_datetime_int_str() -> str:
    current_time = datetime.now()
    # 将当前时间格式化为字符串，精确到毫秒
    time_string = current_time.strftime('%Y-%m-%d %H:%M:%S.%f')
    return "{}{}{}{}{}{}{}".format(
        time_string[0:4], time_string[5:7], time_string[8:10],
        time_string[11:13], time_string[14:16], time_string[17:19], time_string[20:23]
    )


def is_later_than_current_time(time_str: str, minutes: int = 5, current_dt: str = None):
    # 将给定的时间字符串转换为 datetime 对象
    given_time = datetime.strptime(time_str, '%H:%M').time()
    if current_dt is None:
        # 获取当前日期和时间
        current_datetime = datetime.now()
    else:
        current_datetime = datetime.strptime(current_dt, standard_date_format)
    current_time = current_datetime.time()
    # 如果给定时间跨越了当前日期，则调整为明天的时间
    if given_time < current_time:
        given_date = (current_datetime + timedelta(days=1)).date()
    else:
        given_date = current_datetime.date()
    # 合并给定时间的日期和当前日期的时间
    given_datetime = datetime.combine(given_date, given_time)
    # 计算给定时间与当前时间的差值
    time_difference = given_datetime - current_datetime
    # 判断差值是否大于 多少 分钟
    return time_difference.total_seconds() > minutes * 60


def encryp_md5(data: str) -> str:
    # 创建一个 hashlib.md5 对象
    md5_hash = hashlib.md5()
    # 将输入的字符串转换为 bytes，并更新 MD5 哈希对象
    md5_hash.update(data.encode('utf-8'))
    # 获取 MD5 值的十六进制表示形式（32 位小写）
    md5_hex_digest = md5_hash.hexdigest()
    return md5_hex_digest


def covert_dict_key_to_lower(d: dict) -> dict:
    result = dict()
    for key, value in d.items():
        if isinstance(key, str):
            key_new = key.lower()
            result[key_new] = value
    return result


def get_html_title(html: str) -> str:
    # 使用正则表达式提取目标字符串
    pattern = '<title>(.*?)</title>'
    match = re.search(pattern, html)
    if match:
        title = match.group(1)
    else:
        title = "Abnormal HTML document structure"
    return title


def gen_sign(departure_city_code: str, arrival_city_code: str, departure_date: str, transaction_id: str) -> str:
    """
    sign = md5(transactionID + 起始地大写三字码+ 降落的大写三字码 + 出发时间yyyy-MM-dd)
    :param departure_city_code:
    :param arrival_city_code:
    :param departure_date:
    :param transaction_id:
    :return:
    """
    return encryp_md5(
        data="{}{}{}{}".format(transaction_id, departure_city_code, arrival_city_code, departure_date)
    )


def generate_random_ipv6():
    # 生成8个四位的16进制数，并将它们组合成IPv6地址
    ipv6 = ':'.join('{:04x}'.format(random.randint(0, 0xffff)) for _ in range(8))
    return ipv6


def generate_random_ipv6_encode() -> str:
    # 生成8个四位的16进制数，并将它们组合成IPv6地址
    seq = to_url_str(data=":")
    ipv6 = seq.join('{:04x}'.format(random.randint(0, 0xffff)) for _ in range(8))
    return ipv6


def dict_to_jsonstr(data: dict) -> str:
    # 将消息体转换为JSON字符串
    body_json = json.dumps(data, ensure_ascii=False)
    # 将JSON字符串编码为字节
    # body_bytes = body_json.encode('utf-8')
    return body_json


def get_dict_lenth(data: dict) -> int:
    # 计算字节码的字节长度
    content_length = len(dict_to_jsonstr(data=data))
    return content_length


def to_url_str(data: str) -> str:
    # 对JSON字符串进行URL编码，确保所有特殊字符被正确编码。
    encoded_data = urllib.parse.quote(data)
    return encoded_data


def is_exists(file_name: t.LiteralString | str | bytes) -> bool:
    if os.path.exists(file_name):
        return True
    else:
        return False


def is_file(file_path: str):
    return os.path.exists(file_path) and os.path.isfile(file_path)


def is_dir(file_path: str):
    return os.path.exists(file_path) and os.path.isdir(file_path)


def get_project_path():
    # 获取当前执行文件的绝对路径（兼容 Python 2 和 Python 3）
    exec_file_path = os.path.abspath(sys.argv[0])
    exec_file_path_slice = exec_file_path.split(os.path.sep)
    return os.path.sep.join(exec_file_path_slice[:-1])


def join_path(path_slice: list) -> t.LiteralString | str | bytes:
    return os.path.join(*path_slice)


def create_directory(dir_path):
    """
    创建目录，如果目录不存在则创建
    :param dir_path: 要创建的目录路径
    :return: True（创建成功）或 False（创建失败）
    """
    try:
        os.makedirs(dir_path, exist_ok=True)
        return True
    except OSError:
        return False


def get_image_dir() -> t.LiteralString | str | bytes:
    """
    获取image目录
    :return:
    """
    path = join_path([get_project_path(), "image"])
    path = path if isinstance(path, str) else str(path)
    if is_dir(file_path=path) is False:
        create_directory(dir_path=path)
    return path


def get_log_dir() -> t.LiteralString | str | bytes:
    """
    获取日志目录
    :return:
    """
    path = join_path([get_project_path(), "log"])
    path = path if isinstance(path, str) else str(path)
    if is_dir(file_path=path) is False:
        create_directory(dir_path=path)
    return path



# -*- coding: utf-8 -*-
"""
# ---------------------------------------------------------------------------------------------------------
# ProjectName:  ctrip-helper
# FileName:     http_api.py
# Description:  TODO
# Author:       mfkifhss2023
# CreateDate:   2024/05/26
# Copyright ©2011-2024. Hunan xxxxxxx Company limited. All rights reserved.
# ---------------------------------------------------------------------------------------------------------
"""
import json
import random
from copy import deepcopy
from ctrip_helper.http_client import HttpService
from ctrip_helper.config import url_map, common_headers
from ctrip_helper.utils import gen_sign, generate_random_ipv6_encode, get_dict_lenth, dict_to_jsonstr, to_url_str

"""
# 设置代理 example
proxies = {
    'http': 'http://10.10.1.10:3128',
    'https': 'https://10.10.1.10:1080',
}
"""


class Api(object):
    __headers = {
        'dnt': '1',
        'referer': 'https://verify.ctrip.com/',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) ' +
                      'Chrome/102.0.0.0 Safari/537.36'
    }

    def __init__(self):
        self.__domain = url_map.get("domain")
        self.__protocol = url_map.get("protocol")
        self.__http_client = HttpService(self.__domain, self.__protocol)

    def get_city_code(self, headers: dict = None, proxies: dict = None):
        """获取城市编码"""
        city_name, code = [], []
        if headers:
            self.__headers = headers
        r = self.__http_client.send_request(
            method="get", path=url_map.get("city_code"), headers=self.__headers, params=dict(v=str(random.random())),
            proxies=proxies,
        )
        city_s = r.get('data')
        for city in city_s:
            if city == '热门':
                continue
            for key in city:
                try:
                    for k in city_s[city][key]:
                        city_name.append(k['display'])
                        code.append(k['data'])
                except (Exception,):
                    continue
        city_code = dict(zip(city_name, code))

        return city_name, city_code

    def get_international_holiday(self, headers: dict = None, proxies: dict = None) -> dict:
        if headers:
            self.__headers = headers
        return self.__http_client.send_request(
            method="get", path=url_map.get("international_holiday"), headers=self.__headers,
            params=dict(v=str(random.random())),
            proxies=proxies,
        )

    def get_country_code(self, headers: dict = None, proxies: dict = None) -> dict:
        if headers:
            self.__headers = headers
        result = self.__http_client.send_request(
            method="post", path=url_map.get("flight_city"), headers=self.__headers, params=dict(v=str(random.random())),
            proxies=proxies, json={"DataSearchType": 2}
        )
        if result.get("data"):
            data = json.loads(result.get("data"))
            data = {x.get("areaName"): x for x in data}
        else:
            data = dict()
        return data

    def get_internal_airline(self, headers: dict = None, proxies: dict = None) -> dict:
        """获取国内航线"""
        if headers:
            self.__headers = headers
        result = self.__http_client.send_request(
            method="post", path=url_map.get("flight_city"), headers=self.__headers, params=dict(v=str(random.random())),
            proxies=proxies, json={"DataSearchType": 2}
        )
        if result.get("data"):
            data = json.loads(result.get("data"))
        else:
            data = dict()
        return data

    def get_flight_segments(
            self, departure_city_code: str, arrival_city_code: str, departure_date: str, headers: dict = None,
            proxies: dict = None
    ) -> dict:
        if headers:
            self.__headers = headers
        path = "-".join([url_map.get("flight_segments"), departure_city_code.lower(), arrival_city_code.lower()])
        params = {
            "_": 1,
            "depdate": departure_date,  # 起飞时间
            "cabin": "Y_M_S_C_J_F",  # 查询舱位, Y,M:经济舱,S,C:特价舱位,J:商务舱，"C_F"，"Y_S"
            "adult": 1,  # 是否包含成人
            "child": 0,  # 是否包含儿童
            "infant": 0,  # 是否包含婴儿
            "containstax": 1  # 金额是否含税
        }
        result = self.__http_client.send_request(
            method="get", path=path, headers=self.__headers, params=params, proxies=proxies
        )
        if result.get("status") == 0 and result.get("data"):
            data = result.get("data")
        else:
            data = dict()
        return data

    def get_airline_list(
            self, departure_city_code: str, arrival_city_code: str, departure_date: str, headers: dict = None,
            proxies: dict = None, cookie: str = None, token: str = None
    ) -> list:
        flight_segments_result = self.get_flight_segments(
            departure_city_code=departure_city_code, arrival_city_code=arrival_city_code, departure_date=departure_date,
            headers=headers, proxies=proxies
        )
        data = list()
        if flight_segments_result:
            if headers:
                self.__headers = headers
            scope = flight_segments_result.get("scope")
            transaction_id = flight_segments_result.get("transactionID")
            flight_segment = flight_segments_result.get("flightSegments")[0]
            sign = gen_sign(
                departure_date=departure_date, arrival_city_code=arrival_city_code,
                departure_city_code=departure_city_code, transaction_id=transaction_id
            )
            referer = "https://flights.ctrip.com/online/list/oneway-{}-{}?_=1&depdate={}&sortByPrice=true".format(
                departure_city_code.lower(), arrival_city_code.lower(), departure_date
            )
            common_headers_new = deepcopy(common_headers)
            common_headers_new["sign"] = headers.get("sign") if headers else sign
            common_headers_new["scope"] = headers.get("scope") if headers else scope
            common_headers_new["referer"] = headers.get("referer") if headers else referer
            common_headers_new["transactionid"] = headers.get("transactionid") if headers else transaction_id
            if not cookie and not headers:
                arrival_city_id = flight_segment.get("arrivalCityId")
                arrival_city_name = flight_segment.get("arrivalCityName")
                departure_city_id = flight_segment.get("departureCityId")
                departure_city_name = flight_segment.get("departureCityName")
                arrival_city_time_zone = flight_segment.get("arrivalCityTimeZone")
                departure_city_time_zone = flight_segment.get("departureCityTimeZone")
                # ["SZX|深圳(SZX)|30|SZX|480","CGO|郑州(CGO)|559|CGO|480","2024-06-10"]
                quota_str = to_url_str(data='"')
                comma_str = to_url_str(data=',')
                departure_city_encode = '{}{}|{}({})|{}|{}|{}{}{}'.format(
                    quota_str, departure_city_code, to_url_str(data=departure_city_name), departure_city_code,
                    departure_city_id, departure_city_code, departure_city_time_zone, quota_str, comma_str
                )
                arrival_city_encode = '{}{}|{}({})|{}|{}|{}{}{}'.format(
                    quota_str, arrival_city_code, to_url_str(data=arrival_city_name), arrival_city_code,
                    arrival_city_id, arrival_city_code, arrival_city_time_zone, quota_str, comma_str
                )
                data_encode = '{}{}{}'.format(quota_str, departure_date, quota_str)
                search = "[" + "".join([departure_city_encode, arrival_city_encode, data_encode]) + "]"
                cookie = ('UBT_VID=1716362617552.9853FAAvasqr; GUID=09031026416291311476; _RSG=6mq9jgQBNZA7MjC' +
                          'Gs2syWA; _RDG=28c797987f59e226233840389a89d51ba0; _RGUID=a8d72daa-ad27-4c06-a83c-' +
                          'd82d9032d587; nfes_isSupportWebP=1; MKT_CKID=1716642869118.5mwd3.h3jr; _jzqco=%7C%7C%7C%7C' +
                          '1716642869353%7C1.1476965701.1716642869125.1716642869125.1716642869125.1716642869125.' +
                          '1716642869125.0.0.0.1.1; FlightIntl=Search={};'.format(search)
                          + ' _RF1={}; _bfa=1.1716362617552.9853FAAvasqr.'.format(generate_random_ipv6_encode()) +
                          '1.1716862756271.1716862767381.9.2.10320673302')
                common_headers_new["cookie"] = cookie
            if not token and not headers:
                token = ("1001-common-D3kx4bEMPKlUj90jFZjABKctw0pYUSwanwXTw7SInLjUmEQPR71RFPv58jBpWfpEpNiBHiZsy" +
                         "M4ya1E6zjBtJcpW10vQ7jBcWZnvSdjDtEM4K9ZRfGj5YhnjGpJGyTORa6yBFRAUw51jTbvnpWBlYLaRfsW0U" +
                         "jMDYU9wGQYOnwLFeSYb1JpBy3JXbENXJPlrLYlPvFoYgAWnURkMy01ROnwt4jANjQUjo1Rb1yOPjAhWTGinBi" +
                         "f8I07jF3YDgwBfY7FRGkEXqiATjSUETajZOwN0W6aIaJttxlYzLyOPj3hWbFyTdEakvDMJDXj3Mw05RBAysAe" +
                         "ZbvNFyHcWMaJOqeAgEasi0gJQoj7FeP3vHFi6kYabvOpeN7EthiMbjhPyOlvO5vU1Wzbi7ZwcaJL1YAziO8" +
                         "w1LJ7qwtYbbxF7R3bihaymgYcfYoXwUoyfNvtUY56wP4R9DIqZISYgAvocRlcwT1Y8GEHajs4E09i3YaoYUHR" +
                         "lLrO8vFsenUYc9igzYbdwsaYSXjDY9ay3figmW0ae3fEDGjM4W0FEPSe5fjsYstIfmWqlJgor3gKNleD1EFQW" +
                         "tOI9ZYsElYPsyUtKhbEHMRT7vBaYpfWd3eB8Rh7WZbjQqW86JzBYgOrfY65ifkJaXeSmRqt" +
                         "vlzYMnW94eBcRZSWc3EnmWzGyQ3Ea5")
                common_headers_new["token"] = token
            # OW 单程，RT 往返，MT 多程
            flight_segments_result["flightWayEnum"] = "OW"
            # 直飞
            flight_segments_result["directFlight"] = True
            common_headers_new["content-length"] = str(get_dict_lenth(data=flight_segments_result))
            self.__headers.update(common_headers_new)
            result = self.__http_client.send_request(
                method="post", path=url_map.get("airline_list"), headers=self.__headers,
                params=dict(v=str(random.random())), proxies=proxies, data=dict_to_jsonstr(flight_segments_result)
            )
            if result.get("status") == 0 and isinstance(result.get("data"), dict):
                data = result.get("data").get("flightItineraryList") or list()
        return data
